@@ -1,7 +1,7 @@
 // qjackctl.cpp
 //
 /****************************************************************************
-   Copyright (C) 2003-2021, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2022, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -32,10 +32,15 @@
 #include <QLibraryInfo>
 #include <QTranslator>
 #include <QLocale>
-#include <QCoreApplication>
 
 #include <QSessionManager>
 
+#if QT_VERSION >= QT_VERSION_CHECK(5, 2, 0)
+#ifdef CONFIG_JACK_VERSION
+#include <jack/jack.h>
+#endif
+#endif
+
 #if QT_VERSION < QT_VERSION_CHECK(4, 5, 0)
 namespace Qt {
 const WindowFlags WindowCloseButtonHint = WindowFlags(0x08000000);
@@ -46,6 +51,10 @@ const WindowFlags WindowCloseButtonHint = WindowFlags(0x08000000);
 #define CONFIG_PREFIX	"/usr/local"
 #endif
 
+#ifndef CONFIG_BINDIR
+#define CONFIG_BINDIR	CONFIG_PREFIX "/bin"
+#endif
+
 #ifndef CONFIG_DATADIR
 #define CONFIG_DATADIR	CONFIG_PREFIX "/share"
 #endif
@@ -100,6 +109,21 @@ qjackctlApplication::qjackctlApplication ( int& argc, char **argv )
 	QApplication::setApplicationName(QJACKCTL_TITLE);
 	QApplication::setApplicationDisplayName(QJACKCTL_TITLE);
 	//	QJACKCTL_TITLE " - " + QObject::tr(QJACKCTL_SUBTITLE));
+#if QT_VERSION >= QT_VERSION_CHECK(5, 7, 0)
+	QApplication::setDesktopFileName(
+		QString("org.rncbc.%1").arg(PACKAGE_TARNAME));
+#endif
+	QString sVersion(CONFIG_BUILD_VERSION);
+	sVersion += '\n';
+	sVersion += QString("Qt: %1").arg(qVersion());
+#if defined(QT_STATIC)
+	sVersion += "-static";
+#endif
+	sVersion += '\n';
+#ifdef CONFIG_JACK_VERSION
+	sVersion += tr("JACK: %1").arg(jack_get_version_string());
+#endif
+	QApplication::setApplicationVersion(sVersion);
 #endif
 	// Load translation support.
 	QLocale loc;
@@ -130,11 +154,13 @@ qjackctlApplication::qjackctlApplication ( int& argc, char **argv )
 		if (m_pMyTranslator->load(sLocName, sLocPath)) {
 			QApplication::installTranslator(m_pMyTranslator);
 		} else {
-#ifndef _WIN32
-			sLocPath = CONFIG_DATADIR "/qjackctl/translations";
-#else
-			sLocPath = QCoreApplication::applicationDirPath() + "/translations";
-#endif
+			sLocPath = QApplication::applicationDirPath();
+		#ifndef _WIN32
+			sLocPath.remove(CONFIG_BINDIR);
+			sLocPath.append(CONFIG_DATADIR "/qjackctl/translations");
+		#else
+			sLocPath.append("/translations");
+		#endif
 			if (m_pMyTranslator->load(sLocName, sLocPath)) {
 				QApplication::installTranslator(m_pMyTranslator);
 			} else {
@@ -214,6 +240,13 @@ bool qjackctlApplication::setup ( const QString& sServerName )
 	m_pDisplay = QX11Info::display();
 	if (m_pDisplay) {
 		QString sUnique = QJACKCTL_XUNIQUE;
+		QString sUserName = QString::fromUtf8(::getenv("USER"));
+		if (sUserName.isEmpty())
+			sUserName = QString::fromUtf8(::getenv("USERNAME"));
+		if (!sUserName.isEmpty()) {
+			sUnique += ':';
+			sUnique += sUserName;
+		}
 		if (sServerName.isEmpty()) {
 			const char *pszServerName = ::getenv("JACK_DEFAULT_SERVER");
 			if (pszServerName && ::strcmp("default", pszServerName)) {
@@ -278,6 +311,13 @@ bool qjackctlApplication::setup ( const QString& sServerName )
 		m_sUnique += '-';
 		m_sUnique += sServerName;
 	}
+	QString sUserName = QString::fromUtf8(::getenv("USER"));
+	if (sUserName.isEmpty())
+		sUserName = QString::fromUtf8(::getenv("USERNAME"));
+	if (!sUserName.isEmpty()) {
+		m_sUnique += ':';
+		m_sUnique += sUserName;
+	}
 	m_sUnique += '@';
 	m_sUnique += QHostInfo::localHostName();
 #ifdef Q_OS_UNIX
@@ -527,7 +567,7 @@ int main ( int argc, char **argv )
 	::signal(SIGBUS,  stacktrace);
 #endif
 #endif
-#if defined(Q_OS_LINUX)
+#if defined(Q_OS_LINUX) && !defined(CONFIG_WAYLAND)
 	::setenv("QT_QPA_PLATFORM", "xcb", 0);
 #endif
 #if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
