@@ -1,7 +1,7 @@
 // qjackctlAboutForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2003-2020, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2022, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -38,7 +38,9 @@ qjackctlAboutForm::qjackctlAboutForm ( QWidget *pParent )
 {
 	// Setup UI struct...
 	m_ui.setupUi(this);
-
+#if QT_VERSION < QT_VERSION_CHECK(6, 1, 0)
+	QDialog::setWindowIcon(QIcon(":/images/qjackctl.png"));
+#endif
 	QStringList list;
 #ifdef CONFIG_DEBUG
 	list << tr("Debugging option enabled.");
