@@ -1,7 +1,7 @@
 // qjackctlGraph.cpp
 //
 /****************************************************************************
-   Copyright (C) 2003-2021, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2022, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -44,6 +44,7 @@
 
 #include <QGraphicsProxyWidget>
 #include <QLineEdit>
+#include <QScrollBar>
 
 #include <QMouseEvent>
 #include <QWheelEvent>
@@ -972,11 +973,6 @@ qjackctlGraphConnect::qjackctlGraphConnect (void)
 // Destructor.
 qjackctlGraphConnect::~qjackctlGraphConnect (void)
 {
-	if (m_port1)
-		m_port1->removeConnect(this);
-	if (m_port2)
-		m_port2->removeConnect(this);
-
 	// No actual need to destroy any children here...
 	//
 	//QGraphicsPathItem::setGraphicsEffect(nullptr);
@@ -1026,6 +1022,21 @@ qjackctlGraphPort *qjackctlGraphConnect::port2 (void) const
 }
 
 
+// Active disconnection.
+void qjackctlGraphConnect::disconnect (void)
+{
+	if (m_port1) {
+		m_port1->removeConnect(this);
+		m_port1 = nullptr;
+	}
+
+	if (m_port2) {
+		m_port2->removeConnect(this);
+		m_port2 = nullptr;
+	}
+}
+
+
 // Path/shaper updaters.
 void qjackctlGraphConnect::updatePathTo ( const QPointF& pos )
 {
@@ -1042,12 +1053,23 @@ void qjackctlGraphConnect::updatePathTo ( const QPointF& pos )
 
 	qjackctlGraphNode *node1 = m_port1->portNode();
 	const QRectF& rect1 = node1->itemRect();
+	const qreal h1 = 0.5 * rect1.height();
+	const qreal dh = pos0.y() - node1->scenePos().y() - h1;
 	const qreal dx = pos3_4.x() - pos1_2.x();
-	const qreal dy = pos0.y() - node1->scenePos().y() - 0.5 * rect1.height();
-	const qreal y_max = rect1.height() + rect1.width();
-	const qreal y_min = qMin(y_max, qAbs(dx));
-	const qreal x_offset = (dx > 0.0 ? 0.5 : 1.5) * y_min;
-	const qreal y_offset = (dx > 0.0 ? 0.0 : (dy > 0.0 ? +y_min : -y_min));
+	const qreal x_max = rect1.width() + h1;
+	const qreal x_min = qMin(x_max, qAbs(dx));
+	const qreal x_offset = (dx > 0.0 ? 0.5 : 1.0) * x_min;
+
+	qreal y_offset = 0.0;
+	if (g_connect_through_nodes) {
+		// New "normal" connection line curves (inside/through nodes)...
+		const qreal h2 = m_port1->itemRect().height();
+		const qreal dy = qAbs(pos3_4.y() - pos1_2.y());
+		y_offset = (dx > -h2 || dy > h2 ? 0.0 : (dh > 0.0 ? +h2 : -h2));
+	} else {
+		// Old "weird" connection line curves (outside/around nodes)...
+		y_offset = (dx > 0.0 ? 0.0 : (dh > 0.0 ? +x_min : -x_min));
+	}
 
 	const QPointF pos2(pos1.x() + x_offset, pos1.y() + y_offset);
 	const QPointF pos3(pos4.x() - x_offset, pos4.y() + y_offset);
@@ -1078,7 +1100,8 @@ void qjackctlGraphConnect::updatePathTo ( const QPointF& pos )
 
 void qjackctlGraphConnect::updatePath (void)
 {
-	updatePathTo(m_port2->portPos());
+	if (m_port2)
+		updatePathTo(m_port2->portPos());
 }
 
 
@@ -1184,6 +1207,21 @@ void qjackctlGraphConnect::updatePortTypeColors (void)
 }
 
 
+// Connector curve draw style (through vs. around nodes)
+//
+bool qjackctlGraphConnect::g_connect_through_nodes = false;
+
+void qjackctlGraphConnect::setConnectThroughNodes ( bool on )
+{
+	g_connect_through_nodes = on;
+}
+
+bool qjackctlGraphConnect::isConnectThroughNodes (void)
+{
+	return g_connect_through_nodes;
+}
+
+
 //----------------------------------------------------------------------------
 // qjackctlGraphCanvas -- Canvas graphics scene/view.
 
@@ -1203,8 +1241,8 @@ qjackctlGraphCanvas::qjackctlGraphCanvas ( QWidget *parent )
 		m_connect(nullptr), m_rubberband(nullptr),
 		m_zoom(1.0), m_zoomrange(false),
 		m_commands(nullptr), m_settings(nullptr),
-		m_selected_nodes(0), m_edit_item(nullptr),
-		m_editor(nullptr), m_edited(0),
+		m_selected_nodes(0), m_repel_overlapping_nodes(false),
+		m_edit_item(nullptr), m_editor(nullptr), m_edited(0),
 		m_aliases(nullptr)
 {
 	m_scene = new QGraphicsScene();
@@ -1281,7 +1319,9 @@ void qjackctlGraphCanvas::addItem ( qjackctlGraphItem *item )
 		if (node) {
 			m_nodes.append(node);
 			m_nodekeys.insert(qjackctlGraphNode::NodeKey(node), node);
-			if (!restoreNodePos(node))
+			if (restoreNode(node))
+				emit updated(node);
+			else
 				emit added(node);
 		}
 	}
@@ -1294,7 +1334,7 @@ void qjackctlGraphCanvas::removeItem ( qjackctlGraphItem *item )
 
 	if (item->type() == qjackctlGraphNode::Type) {
 		qjackctlGraphNode *node = static_cast<qjackctlGraphNode *> (item);
-		if (node && saveNodePos(node)) {
+		if (node && saveNode(node)) {
 			emit removed(node);
 			node->removePorts();
 			m_nodekeys.remove(qjackctlGraphNode::NodeKey(node));
@@ -1314,11 +1354,17 @@ qjackctlGraphItem *qjackctlGraphCanvas::currentItem (void) const
 {
 	qjackctlGraphItem *item = m_item;
 
-	if (item == nullptr) {
-		const QList<QGraphicsItem *>& list
-			= m_scene->selectedItems();
-		if (!list.isEmpty())
-			item = static_cast<qjackctlGraphItem *> (list.first());
+	if (item && item->type() == qjackctlGraphConnect::Type)
+		item = nullptr;
+
+ 	if (item == nullptr) {
+		foreach (QGraphicsItem *item2, m_scene->selectedItems()) {
+			if (item2->type() == qjackctlGraphConnect::Type)
+				continue;
+			item = static_cast<qjackctlGraphItem *> (item2);
+			if (item2->type() == qjackctlGraphNode::Type)
+				break;
+		}
 	}
 
 	return item;
@@ -1482,6 +1528,13 @@ qjackctlGraphNode *qjackctlGraphCanvas::findNode (
 }
 
 
+// Whether it's in the middle of something...
+bool qjackctlGraphCanvas::isBusy (void) const
+{
+	return (m_state != DragNone || m_connect   != nullptr
+		||  m_item  != nullptr  || m_edit_item != nullptr);
+}
+
 
 // Port (dis)connections notifiers.
 void qjackctlGraphCanvas::emitConnected (
@@ -1524,12 +1577,13 @@ qjackctlGraphItem *qjackctlGraphCanvas::itemAt ( const QPointF& pos ) const
 void qjackctlGraphCanvas::connectPorts (
 	qjackctlGraphPort *port1, qjackctlGraphPort *port2, bool is_connect )
 {
+#if 0 // Sure the sect will check to this instead...?
 	const bool is_connected // already connected?
 		= (port1->findConnect(port2) != nullptr);
 	if (( is_connect &&  is_connected) ||
 		(!is_connect && !is_connected))
 		return;
-
+#endif
 	if (port1->isOutput()) {
 		m_commands->push(
 			new qjackctlGraphConnectCommand(this, port1, port2, is_connect));
@@ -1551,14 +1605,16 @@ void qjackctlGraphCanvas::mousePressEvent ( QMouseEvent *event )
 	if (item && item->type() >= QGraphicsItem::UserType)
 		m_item = static_cast<qjackctlGraphItem *> (item);
 
-	if (event->button() == Qt::LeftButton)
+	if (event->button() == Qt::LeftButton ||
+		event->button() == Qt::MiddleButton)
 		m_state = DragStart;
 
 	if (m_state == DragStart && m_item == nullptr
-		&& (event->modifiers() & Qt::ControlModifier)
+		&& (((event->button() == Qt::LeftButton)
+		  && (event->modifiers() & Qt::ControlModifier))
+		  || (event->button() == Qt::MiddleButton))
 		&& m_scene->selectedItems().isEmpty()) {
-		QGraphicsView::setDragMode(QGraphicsView::ScrollHandDrag);
-		QGraphicsView::mousePressEvent(event);
+		QGraphicsView::setCursor(Qt::ClosedHandCursor);
 		m_state = DragScroll;
 	}
 }
@@ -1713,9 +1769,16 @@ void qjackctlGraphCanvas::mouseMoveEvent ( QMouseEvent *event )
 			 m_connect->setZValue(zval);
 		}
 		break;
-	case DragScroll:
+	case DragScroll: {
+		QScrollBar *hbar = QGraphicsView::horizontalScrollBar();
+		QScrollBar *vbar = QGraphicsView::verticalScrollBar();
+		const QPoint delta = (pos - m_pos).toPoint();
+		hbar->setValue(hbar->value() - delta.x());
+		vbar->setValue(vbar->value() - delta.y());
+		m_pos = pos;
+		break;
+	}
 	default:
-		QGraphicsView::mouseMoveEvent(event);
 		break;
 	}
 
@@ -1764,9 +1827,11 @@ void qjackctlGraphCanvas::mouseReleaseEvent ( QMouseEvent *event )
 					&& port1->portType() == port2->portType()
 					&& port1->findConnect(port2) == nullptr) {
 					port2->setSelected(true);
-				#if 0 // Sure the sect will commit to this instead...
+				#if 1 // Sure the sect will commit to this instead...?
 					m_connect->setPort2(port2);
+					m_connect->updatePortTypeColors();
 					m_connect->updatePathTo(port2->portPos());
+					emit connected(m_connect);
 					m_connect = nullptr;
 					++m_selected_nodes;
 				#else
@@ -1781,8 +1846,11 @@ void qjackctlGraphCanvas::mouseReleaseEvent ( QMouseEvent *event )
 				}
 			}
 			// Done with the hovering connection...
-			delete m_connect;
-			m_connect = nullptr;
+			if (m_connect) {
+				m_connect->disconnect();
+				delete m_connect;
+				m_connect = nullptr;
+			}
 		}
 		// Maybe some node(s) were moved...
 		if (m_item && m_item->type() == qjackctlGraphNode::Type) {
@@ -1815,8 +1883,6 @@ void qjackctlGraphCanvas::mouseReleaseEvent ( QMouseEvent *event )
 		break;
 	case DragScroll:
 	default:
-		QGraphicsView::mouseReleaseEvent(event);
-		QGraphicsView::setDragMode(QGraphicsView::NoDrag);
 		break;
 	}
 
@@ -2074,6 +2140,9 @@ void qjackctlGraphCanvas::renameItem (void)
 	}
 	else return;
 
+	m_selected_nodes = 0;
+	m_scene->clearSelection();
+
 	m_editor->show();
 	m_editor->setEnabled(true);
 	m_editor->selectAll();
@@ -2142,6 +2211,19 @@ void qjackctlGraphCanvas::updateNodes (void)
 }
 
 
+// Update all connectors.
+void qjackctlGraphCanvas::updateConnects (void)
+{
+	foreach (QGraphicsItem *item, m_scene->items()) {
+		if (item->type() == qjackctlGraphConnect::Type) {
+			qjackctlGraphConnect *connect = static_cast<qjackctlGraphConnect *> (item);
+			if (connect)
+				connect->updatePath();
+		}
+	}
+}
+
+
 // Zoom in rectangle range.
 void qjackctlGraphCanvas::zoomFitRange ( const QRectF& range_rect )
 {
@@ -2171,7 +2253,7 @@ void qjackctlGraphCanvas::zoomFitRange ( const QRectF& range_rect )
 
 
 // Graph node position methods.
-bool qjackctlGraphCanvas::restoreNodePos ( qjackctlGraphNode *node )
+bool qjackctlGraphCanvas::restoreNode ( qjackctlGraphNode *node )
 {
 	if (m_settings == nullptr || node == nullptr)
 		return false;
@@ -2189,7 +2271,7 @@ bool qjackctlGraphCanvas::restoreNodePos ( qjackctlGraphNode *node )
 }
 
 
-bool qjackctlGraphCanvas::saveNodePos ( qjackctlGraphNode *node ) const
+bool qjackctlGraphCanvas::saveNode ( qjackctlGraphNode *node ) const
 {
 	if (m_settings == nullptr || node == nullptr)
 		return false;
@@ -2352,6 +2434,7 @@ void qjackctlGraphCanvas::clear (void)
 		m_selected.clear();
 	}
 	if (m_connect) {
+		m_connect->disconnect();
 		delete m_connect;
 		m_connect = nullptr;
 	}
@@ -2426,9 +2509,10 @@ qjackctlGraphCanvas *qjackctlGraphSect::canvas (void) const
 
 
 // Generic sect/graph methods.
-void qjackctlGraphSect::addItem ( qjackctlGraphItem *item )
+void qjackctlGraphSect::addItem ( qjackctlGraphItem *item, bool is_new )
 {
-	m_canvas->addItem(item);
+	if (is_new)
+		m_canvas->addItem(item);
 
 	if (item->type() == qjackctlGraphConnect::Type) {
 		qjackctlGraphConnect *connect = static_cast<qjackctlGraphConnect *> (item);
@@ -2442,8 +2526,10 @@ void qjackctlGraphSect::removeItem ( qjackctlGraphItem *item )
 {
 	if (item->type() == qjackctlGraphConnect::Type) {
 		qjackctlGraphConnect *connect = static_cast<qjackctlGraphConnect *> (item);
-		if (connect)
+		if (connect) {
+			connect->disconnect();
 			m_connects.removeAll(connect);
+		}
 	}
 
 	m_canvas->removeItem(item);
@@ -2534,4 +2620,95 @@ void qjackctlGraphSect::renameItem (
 }
 
 
+// Repel overlapping nodes...
+void qjackctlGraphCanvas::setRepelOverlappingNodes ( bool on )
+{
+	m_repel_overlapping_nodes = on;
+}
+
+
+bool qjackctlGraphCanvas::isRepelOverlappingNodes (void) const
+{
+	return m_repel_overlapping_nodes;
+}
+
+
+void qjackctlGraphCanvas::repelOverlappingNodes ( qjackctlGraphNode *node,
+	qjackctlGraphMoveCommand *move_command, const QPointF& delta )
+{
+	const qreal MIN_NODE_GAP = 8.0f;
+
+	node->setMarked(true);
+
+	QRectF rect1 = node->sceneBoundingRect();
+	rect1.adjust(
+		-2.0 * MIN_NODE_GAP, -MIN_NODE_GAP,
+		+2.0 * MIN_NODE_GAP, +MIN_NODE_GAP);
+
+	foreach (qjackctlGraphNode *node2, m_nodes) {
+		if (node2->isMarked())
+			continue;
+		const QPointF& pos1
+			= node2->pos();
+		QPointF pos2 = pos1;
+		const QRectF& rect2
+			= node2->sceneBoundingRect();
+		const QRectF& recti
+			= rect2.intersected(rect1);
+		if (!recti.isNull()) {
+			const QPointF delta2
+				= (delta.isNull() ? rect2.center() - rect1.center() : delta);
+			if (recti.width() < (1.5 * recti.height())) {
+				qreal dx = recti.width();
+				if ((delta2.x() < 0.0 && recti.width() >= rect1.width()) ||
+					(delta2.x() > 0.0 && recti.width() >= rect2.width())) {
+					dx += qAbs(rect2.right() - rect1.right());
+				}
+				else
+				if ((delta2.x() > 0.0 && recti.width() >= rect1.width()) ||
+					(delta2.x() < 0.0 && recti.width() >= rect2.width())) {
+					dx += qAbs(rect2.left() - rect1.left());
+				}
+				if (delta2.x() < 0.0)
+					pos2.setX(pos1.x() - dx);
+				else
+					pos2.setX(pos1.x() + dx);
+			} else {
+				qreal dy = recti.height();
+				if ((delta2.y() < 0.0 && recti.height() >= rect1.height()) ||
+					(delta2.y() > 0.0 && recti.height() >= rect2.height())) {
+					dy += qAbs(rect2.bottom() - rect1.bottom());
+				}
+				else
+				if ((delta2.y() > 0.0 && recti.height() >= rect1.height()) ||
+					(delta2.y() < 0.0 && recti.height() >= rect2.height())) {
+					dy += qAbs(rect2.top() - rect1.top());
+				}
+				if (delta2.y() < 0.0)
+					pos2.setY(pos1.y() - dy);
+				else
+					pos2.setY(pos1.y() + dy);
+			}
+			// Repel this node...
+			node2->setPos(pos2);
+			// Add this node for undo/redo...
+			if (move_command)
+				move_command->addItem(node2, pos1, pos2);
+			// Repel this node neighbors, if any...
+			repelOverlappingNodes(node2, move_command, delta2);
+		}
+	}
+
+	node->setMarked(false);
+}
+
+
+void qjackctlGraphCanvas::repelOverlappingNodesAll (
+	qjackctlGraphMoveCommand *move_command )
+{
+	foreach (qjackctlGraphNode *node, m_nodes)
+		repelOverlappingNodes(node, move_command);
+}
+
+
 // end of qjackctlGraph.cpp
