@@ -39,6 +39,7 @@ class qjackctlGraphNode;
 class qjackctlGraphPort;
 class qjackctlGraphConnect;
 class qjackctlGraphCommand;
+class qjackctlGraphMoveCommand;
 
 class qjackctlAliases;
 class qjackctlAliasList;
@@ -256,6 +257,10 @@ public:
 	// Rectangular editor extents.
 	QRectF editorRect() const;
 
+	// Connector curve draw style (through vs. around nodes)
+	static void setConnectThroughNodes(bool on);
+	static bool isConnectThroughNodes();
+
 protected:
 
 	void paint(QPainter *painter,
@@ -424,6 +429,10 @@ public:
 	// Special port-type color business.
 	void updatePortTypeColors();
 
+	// Connector curve draw style (through vs. around nodes)
+	static void setConnectThroughNodes(bool on);
+	static bool isConnectThroughNodes();
+
 protected:
 
 	void paint(QPainter *painter,
@@ -438,6 +447,9 @@ private:
 	// Instance variables.
 	qjackctlGraphPort *m_port1;
 	qjackctlGraphPort *m_port2;
+
+	// Connector curve draw style (through vs. around nodes)
+	static bool g_connect_through_nodes;
 };
 
 
@@ -506,6 +518,15 @@ public:
 	bool restoreState();
 	bool saveState() const;
 
+	// Repel overlapping nodes...
+	void setRepelOverlappingNodes(bool on);
+	bool isRepelOverlappingNodes() const;
+	void repelOverlappingNodes(qjackctlGraphNode *node,
+		qjackctlGraphMoveCommand *move_command = nullptr,
+		const QPointF& delta = QPointF());
+	void repelOverlappingNodesAll(
+		qjackctlGraphMoveCommand *move_command = nullptr);
+
 	// Graph colors management.
 	void setPortTypeColor(uint port_type, const QColor& color);
 	const QColor& portTypeColor(uint port_type);
@@ -526,12 +547,15 @@ signals:
 
 	// Node factory notifications.
 	void added(qjackctlGraphNode *node);
+	void updated(qjackctlGraphNode *node);
 	void removed(qjackctlGraphNode *node);
 
 	// Port (dis)connection notifications.
 	void connected(qjackctlGraphPort *port1, qjackctlGraphPort *port2);
 	void disconnected(qjackctlGraphPort *port1, qjackctlGraphPort *port2);
 
+	void connected(qjackctlGraphConnect *connect);
+
 	// Generic change notification.
 	void changed();
 
@@ -561,6 +585,9 @@ public slots:
 	// Update all nodes.
 	void updateNodes();
 
+	// Update all connectors.
+	void updateConnects();
+
 protected slots:
 
 	// Rename item slots.
@@ -594,8 +621,8 @@ protected:
 	void zoomFitRange(const QRectF& range_rect);
 
 	// Graph node position state methods.
-	bool restoreNodePos(qjackctlGraphNode *node);
-	bool saveNodePos(qjackctlGraphNode *node) const;
+	bool restoreNode(qjackctlGraphNode *node);
+	bool saveNode(qjackctlGraphNode *node) const;
 
 	// Renaming editor position and size updater.
 	void updateEditorGeometry();
@@ -624,6 +651,8 @@ private:
 	QList<QGraphicsItem *> m_selected;
 	int m_selected_nodes;
 
+	bool m_repel_overlapping_nodes;
+
 	// Graph port colors.
 	QHash<uint, QColor> m_port_colors;
 
@@ -660,7 +689,7 @@ public:
 	qjackctlGraphCanvas *canvas() const;
 
 	// Generic sect/graph methods.
-	void addItem(qjackctlGraphItem *item);
+	void addItem(qjackctlGraphItem *item, bool is_new = true);
 	void removeItem(qjackctlGraphItem *item);
 
 	// Clean-up all un-marked items...
