@@ -1,7 +1,7 @@
 // qjackctlGraphForm.cpp
 //
 /****************************************************************************
-   Copyright (C) 2003-2021, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2003-2022, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -92,6 +92,8 @@ qjackctlGraphForm::qjackctlGraphForm (
 
 	m_ins = m_mids = m_outs = 0;
 
+	m_repel_overlapping_nodes = 0;
+
 	QUndoStack *commands = m_ui.graphCanvas->commands();
 
 	QAction *undo_action = commands->createUndoAction(this, tr("&Undo"));
@@ -158,6 +160,9 @@ qjackctlGraphForm::qjackctlGraphForm (
 	QObject::connect(m_ui.graphCanvas,
 		SIGNAL(added(qjackctlGraphNode *)),
 		SLOT(added(qjackctlGraphNode *)));
+	QObject::connect(m_ui.graphCanvas,
+		SIGNAL(updated(qjackctlGraphNode *)),
+		SLOT(updated(qjackctlGraphNode *)));
 	QObject::connect(m_ui.graphCanvas,
 		SIGNAL(removed(qjackctlGraphNode *)),
 		SLOT(removed(qjackctlGraphNode *)));
@@ -169,6 +174,10 @@ qjackctlGraphForm::qjackctlGraphForm (
 		SIGNAL(disconnected(qjackctlGraphPort *, qjackctlGraphPort *)),
 		SLOT(disconnected(qjackctlGraphPort *, qjackctlGraphPort *)));
 
+	QObject::connect(m_ui.graphCanvas,
+		SIGNAL(connected(qjackctlGraphConnect *)),
+		SLOT(connected(qjackctlGraphConnect *)));
+
 	QObject::connect(m_ui.graphCanvas,
 		SIGNAL(renamed(qjackctlGraphItem *, const QString&)),
 		SLOT(renamed(qjackctlGraphItem *, const QString&)));
@@ -181,6 +190,17 @@ qjackctlGraphForm::qjackctlGraphForm (
 	// shortcuts firmly attached...
 	addAction(m_ui.viewMenubarAction);
 
+	// HACK: Make old Ins/Del standard shortcuts
+	// for connect/disconnect available again...
+	QList<QKeySequence> shortcuts;
+	shortcuts.append(m_ui.graphConnectAction->shortcut());
+	shortcuts.append(QKeySequence("Ins"));
+	m_ui.graphConnectAction->setShortcuts(shortcuts);
+	shortcuts.clear();
+	shortcuts.append(m_ui.graphDisconnectAction->shortcut());
+	shortcuts.append(QKeySequence("Del"));
+	m_ui.graphDisconnectAction->setShortcuts(shortcuts);
+
 	QObject::connect(m_ui.graphConnectAction,
 		SIGNAL(triggered(bool)),
 		m_ui.graphCanvas, SLOT(connectItems()));
@@ -244,53 +264,59 @@ qjackctlGraphForm::qjackctlGraphForm (
 		SIGNAL(triggered(bool)),
 		SLOT(viewZoomRange(bool)));
 
+	QObject::connect(m_ui.viewRepelOverlappingNodesAction,
+		SIGNAL(triggered(bool)),
+		SLOT(viewRepelOverlappingNodes(bool)));
+	QObject::connect(m_ui.viewConnectThroughNodesAction,
+		SIGNAL(triggered(bool)),
+		SLOT(viewConnectThroughNodes(bool)));
+
 	m_ui.viewColorsJackAudioAction->setData(qjackctlJackGraph::audioPortType());
 	m_ui.viewColorsJackMidiAction->setData(qjackctlJackGraph::midiPortType());
-#ifdef CONFIG_ALSA_SEQ
-	m_ui.viewColorsAlsaMidiAction->setData(qjackctlAlsaGraph::midiPortType());
-#endif
 
 	int iAddSeparator = 0;
 #ifdef CONFIG_JACK_CV
 	m_ui.viewColorsJackCvAction->setData(qjackctlJackGraph::cvPortType());
+	m_ui.viewColorsMenu->insertAction(
+		m_ui.viewColorsResetAction, m_ui.viewColorsJackCvAction);
 	++iAddSeparator;
 #endif
 #ifdef CONFIG_JACK_OSC
 	m_ui.viewColorsJackOscAction->setData(qjackctlJackGraph::oscPortType());
+	m_ui.viewColorsMenu->insertAction(
+		m_ui.viewColorsResetAction, m_ui.viewColorsJackOscAction);
 	++iAddSeparator;
 #endif
 	if (iAddSeparator > 0)
 		m_ui.viewColorsMenu->insertSeparator(m_ui.viewColorsResetAction);
 
+#ifdef CONFIG_ALSA_SEQ
+	m_ui.viewColorsAlsaMidiAction->setData(qjackctlAlsaGraph::midiPortType());
+	m_ui.viewColorsMenu->insertAction(
+		m_ui.viewColorsResetAction, m_ui.viewColorsAlsaMidiAction);
+	m_ui.viewColorsMenu->insertSeparator(m_ui.viewColorsResetAction);
+#endif
+
 	QObject::connect(m_ui.viewColorsJackAudioAction,
 		SIGNAL(triggered(bool)),
 		SLOT(viewColorsAction()));
 	QObject::connect(m_ui.viewColorsJackMidiAction,
 		SIGNAL(triggered(bool)),
 		SLOT(viewColorsAction()));
-#ifdef CONFIG_ALSA_SEQ
-	QObject::connect(m_ui.viewColorsAlsaMidiAction,
-		SIGNAL(triggered(bool)),
-		SLOT(viewColorsAction()));
-#else
-	m_ui.viewColorsMenu->removeAction(m_ui.viewColorsAlsaMidiAction);
-	m_ui.viewColorsAlsaMidiAction->setDisabled(true);
-#endif
 #ifdef CONFIG_JACK_CV
 	QObject::connect(m_ui.viewColorsJackCvAction,
 		SIGNAL(triggered(bool)),
 		SLOT(viewColorsAction()));
-#else
-	m_ui.viewColorsMenu->removeAction(m_ui.viewColorsJackCvAction);
-	m_ui.viewColorsJackCvAction->setDisabled(true);
 #endif
 #ifdef CONFIG_JACK_OSC
 	QObject::connect(m_ui.viewColorsJackOscAction,
 		SIGNAL(triggered(bool)),
 		SLOT(viewColorsAction()));
-#else
-	m_ui.viewColorsMenu->removeAction(m_ui.viewColorsJackOscAction);
-	m_ui.viewColorsJackOscAction->setDisabled(true);
+#endif
+#ifdef CONFIG_ALSA_SEQ
+	QObject::connect(m_ui.viewColorsAlsaMidiAction,
+		SIGNAL(triggered(bool)),
+		SLOT(viewColorsAction()));
 #endif
 	QObject::connect(m_ui.viewColorsResetAction,
 		SIGNAL(triggered(bool)),
@@ -386,6 +412,8 @@ void qjackctlGraphForm::setup ( qjackctlSetup *pSetup )
 
 	m_ui.viewTextBesideIconsAction->setChecked(m_config->isTextBesideIcons());
 	m_ui.viewZoomRangeAction->setChecked(m_config->isZoomRange());
+	m_ui.viewRepelOverlappingNodesAction->setChecked(m_config->isRepelOverlappingNodes());
+	m_ui.viewConnectThroughNodesAction->setChecked(m_config->isConnectThroughNodes());
 
 	const qjackctlGraphPort::SortType sort_type
 		= qjackctlGraphPort::SortType(m_config->sortType());
@@ -422,6 +450,8 @@ void qjackctlGraphForm::setup ( qjackctlSetup *pSetup )
 
 	viewTextBesideIcons(m_config->isTextBesideIcons());
 	viewZoomRange(m_config->isZoomRange());
+	viewRepelOverlappingNodes(m_config->isRepelOverlappingNodes());
+	viewConnectThroughNodes(m_config->isConnectThroughNodes());
 
 	m_ui.graphCanvas->restoreState();
 
@@ -482,6 +512,9 @@ void qjackctlGraphForm::viewRefresh (void)
 	jack_changed();
 	alsa_changed();
 
+	if (m_ui.graphCanvas->isRepelOverlappingNodes())
+		++m_repel_overlapping_nodes; // fake nodes added!
+
 	refresh();
 }
 
@@ -556,6 +589,20 @@ void qjackctlGraphForm::viewSortOrderAction (void)
 }
 
 
+void qjackctlGraphForm::viewRepelOverlappingNodes ( bool on )
+{
+	m_ui.graphCanvas->setRepelOverlappingNodes(on);
+	if (on) ++m_repel_overlapping_nodes;
+}
+
+
+void qjackctlGraphForm::viewConnectThroughNodes ( bool on )
+{
+	qjackctlGraphConnect::setConnectThroughNodes(on);
+	m_ui.graphCanvas->updateConnects();
+}
+
+
 void qjackctlGraphForm::helpAbout (void)
 {
 	qjackctlMainForm *pMainForm = qjackctlMainForm::getInstance();
@@ -621,7 +668,14 @@ void qjackctlGraphForm::added ( qjackctlGraphNode *node )
 
 	node->setPos(x, y);
 
-	stabilize();
+	updated(node);
+}
+
+
+void qjackctlGraphForm::updated ( qjackctlGraphNode */*node*/ )
+{
+	if (m_ui.graphCanvas->isRepelOverlappingNodes())
+		++m_repel_overlapping_nodes;
 }
 
 
@@ -692,6 +746,26 @@ void qjackctlGraphForm::disconnected (
 }
 
 
+void qjackctlGraphForm::connected ( qjackctlGraphConnect *connect )
+{
+	qjackctlGraphPort *port1 = connect->port1();
+	if (port1 == nullptr)
+		return;
+
+	if (qjackctlJackGraph::isPortType(port1->portType())) {
+		if (m_jack)
+			m_jack->addItem(connect, false);
+	}
+#ifdef CONFIG_ALSA_SEQ
+	else
+	if (qjackctlAlsaGraph::isPortType(port1->portType())) {
+		if (m_alsa)
+			m_alsa->addItem(connect, false);
+	}
+#endif
+}
+
+
 // Item renaming slot.
 void qjackctlGraphForm::renamed ( qjackctlGraphItem *item, const QString& name )
 {
@@ -729,11 +803,16 @@ void qjackctlGraphForm::alsa_changed (void)
 // Graph refreshner.
 void qjackctlGraphForm::refresh (void)
 {
+	if (m_ui.graphCanvas->isBusy())
+		return;
+
+	int nchanged = 0;
+
 	if (m_jack_changed > 0) {
 		m_jack_changed = 0;
 		if (m_jack)
 			m_jack->updateItems();
-		stabilize();
+		++nchanged;
 	}
 #ifdef CONFIG_ALSA_SEQ
 	else
@@ -741,9 +820,18 @@ void qjackctlGraphForm::refresh (void)
 		m_alsa_changed = 0;
 		if (m_alsa)
 			m_alsa->updateItems();
-		stabilize();
+		++nchanged;
 	}
 #endif
+
+	if (nchanged > 0)
+		stabilize();
+	else
+	if (m_repel_overlapping_nodes > 0) {
+		m_repel_overlapping_nodes = 0;
+		m_ui.graphCanvas->repelOverlappingNodesAll();
+		stabilize();
+	}
 }
 
 
@@ -804,7 +892,7 @@ void qjackctlGraphForm::orientationChanged ( Qt::Orientation orientation )
 // Context-menu event handler.
 void qjackctlGraphForm::contextMenuEvent ( QContextMenuEvent *pContextMenuEvent )
 {
-//	m_ui.graphCanvas->clear();
+	m_ui.graphCanvas->clear();
 
 	stabilize();
 
@@ -859,6 +947,8 @@ void qjackctlGraphForm::closeEvent ( QCloseEvent *pCloseEvent )
 	m_ui.graphCanvas->saveState();
 
 	if (m_config && QMainWindow::isVisible()) {
+		m_config->setConnectThroughNodes(m_ui.viewConnectThroughNodesAction->isChecked());
+		m_config->setRepelOverlappingNodes(m_ui.viewRepelOverlappingNodesAction->isChecked());
 		m_config->setSortOrder(int(qjackctlGraphPort::sortOrder()));
 		m_config->setSortType(int(qjackctlGraphPort::sortType()));
 		m_config->setTextBesideIcons(m_ui.viewTextBesideIconsAction->isChecked());
@@ -868,6 +958,12 @@ void qjackctlGraphForm::closeEvent ( QCloseEvent *pCloseEvent )
 		m_config->setMenubar(m_ui.MenuBar->isVisible());
 		m_config->saveState(this);
 	}
+	else
+	if (m_repel_overlapping_nodes > 0) {
+		m_repel_overlapping_nodes = 0;
+		m_ui.graphCanvas->repelOverlappingNodesAll();
+		stabilize();
+	}
 
 	QMainWindow::closeEvent(pCloseEvent);
 }
@@ -948,6 +1044,8 @@ static const char *ViewTextBesideIconsKey = "/TextBesideIcons";
 static const char *ViewZoomRangeKey = "/ZoomRange";
 static const char *ViewSortTypeKey  = "/SortType";
 static const char *ViewSortOrderKey = "/SortOrder";
+static const char *ViewRepelOverlappingNodesKey = "/RepelOverlappingNodes";
+static const char *ViewConnectThroughNodesKey = "/ConnectThroughNodes";
 
 
 // Constructors.
@@ -955,7 +1053,8 @@ qjackctlGraphConfig::qjackctlGraphConfig ( QSettings *settings )
 	: m_settings(settings), m_menubar(false),
 		m_toolbar(false), m_statusbar(false),
 		m_texticons(false), m_zoomrange(false),
-		m_sorttype(0), m_sortorder(0)
+		m_sorttype(0), m_sortorder(0),
+		m_repelnodes(false), m_cthrunodes(false)
 {
 }
 
@@ -1043,6 +1142,30 @@ int qjackctlGraphConfig::sortOrder (void) const
 }
 
 
+void qjackctlGraphConfig::setRepelOverlappingNodes ( bool repelnodes )
+{
+	m_repelnodes = repelnodes;
+}
+
+
+bool qjackctlGraphConfig::isRepelOverlappingNodes (void) const
+{
+	return m_repelnodes;
+}
+
+
+void qjackctlGraphConfig::setConnectThroughNodes ( bool cthrunodes )
+{
+	m_cthrunodes = cthrunodes;
+}
+
+
+bool qjackctlGraphConfig::isConnectThroughNodes (void) const
+{
+	return m_cthrunodes;
+}
+
+
 // Graph main-widget state methods.
 bool qjackctlGraphConfig::restoreState ( QMainWindow *widget )
 {
@@ -1057,6 +1180,8 @@ bool qjackctlGraphConfig::restoreState ( QMainWindow *widget )
 	m_zoomrange = m_settings->value(ViewZoomRangeKey, false).toBool();
 	m_sorttype  = m_settings->value(ViewSortTypeKey, 0).toInt();
 	m_sortorder = m_settings->value(ViewSortOrderKey, 0).toInt();
+	m_repelnodes = m_settings->value(ViewRepelOverlappingNodesKey, false).toBool();
+	m_cthrunodes = m_settings->value(ViewConnectThroughNodesKey, false).toBool();
 	m_settings->endGroup();
 
 	m_settings->beginGroup(LayoutGroup);
@@ -1084,6 +1209,8 @@ bool qjackctlGraphConfig::saveState ( QMainWindow *widget ) const
 	m_settings->setValue(ViewZoomRangeKey, m_zoomrange);
 	m_settings->setValue(ViewSortTypeKey, m_sorttype);
 	m_settings->setValue(ViewSortOrderKey, m_sortorder);
+	m_settings->setValue(ViewRepelOverlappingNodesKey, m_repelnodes);
+	m_settings->setValue(ViewConnectThroughNodesKey, m_cthrunodes);
 	m_settings->endGroup();
 
 	m_settings->beginGroup(LayoutGroup);
