@@ -33,6 +33,7 @@ class qjackctlAlsaGraph;
 class qjackctlJackGraph;
 
 class qjackctlGraphPort;
+class qjackctlGraphConnect;
 
 class qjackctlSetup;
 
@@ -82,12 +83,15 @@ protected slots:
 
 	// Node life-cycle slots
 	void added(qjackctlGraphNode *node);
+	void updated(qjackctlGraphNode *node);
 	void removed(qjackctlGraphNode *node);
 
 	// Port (dis)connection slots.
 	void connected(qjackctlGraphPort *port1, qjackctlGraphPort *port2);
 	void disconnected(qjackctlGraphPort *port1, qjackctlGraphPort *port2);
 
+	void connected(qjackctlGraphConnect *connect);
+
 	// Item renaming slot.
 	void renamed(qjackctlGraphItem *item, const QString& name);
 
@@ -115,6 +119,9 @@ protected slots:
 	void viewColorsAction();
 	void viewColorsReset();
 
+	void viewRepelOverlappingNodes(bool on);
+	void viewConnectThroughNodes(bool on);
+
 	void helpAbout();
 	void helpAboutQt();
 
@@ -157,6 +164,8 @@ private:
 
 	int m_ins, m_mids, m_outs;
 
+	int m_repel_overlapping_nodes;
+
 	QSlider  *m_zoom_slider;
 	QSpinBox *m_zoom_spinbox;
 
@@ -199,6 +208,12 @@ public:
 	void setSortOrder(int sortorder);
 	int sortOrder() const;
 
+	void setRepelOverlappingNodes(bool repelnodes);
+	bool isRepelOverlappingNodes() const;
+
+	void setConnectThroughNodes(bool repelnodes);
+	bool isConnectThroughNodes() const;
+
 	// Graph main-widget state methods.
 	bool restoreState(QMainWindow *widget);
 	bool saveState(QMainWindow *widget) const;
@@ -215,6 +230,9 @@ private:
 	bool       m_zoomrange;
 	int        m_sorttype;
 	int        m_sortorder;
+
+	bool       m_repelnodes;
+	bool       m_cthrunodes;
 };
 
 
